import { isEqual, isNil } from 'lodash-es';
import { useCallback, useEffect, useState } from 'react';
import { useCopyToClipboard } from 'react-use';
import { ColorComponentInput } from './ColorComponentInput';
import styles from './ColorPicker.module.css';
import { HSegmentedControl } from './HSegmentedControl';
import { HslAssemble } from './HslAsssemble';
import { NotificationType, useNotification } from './Notifications';
import { RGBAssemble } from './RGBAssemble';

type ColorPickerProps = {
  color?: string | null;
  onSelect?: (color: string) => void;
};

export function ColorPicker({ color, onSelect }: ColorPickerProps) {
  const [cpState, copyToClipboard] = useCopyToClipboard();
  const { showToast } = useNotification();
  const [pickMode, setMode] = useState<'rgb' | 'hsl' | 'input'>('rgb');
  const [selectedColor, setSelectedColor] = useState<string>(color ?? '000000');
  const handleColorSelect = (color: string) => {
    setSelectedColor(color);
    onSelect?.(color);
  };
  const handleCopyAction = useCallback(() => {
    copyToClipboard(`#${selectedColor}`);
  }, [selectedColor]);

  useEffect(() => {
    if (!isNil(cpState.error)) {
      showToast(NotificationType.ERROR, 'Failed to copy to clipboard', 'tabler:alert-circle', 3000);
    } else if (!isNil(cpState.value)) {
      showToast(
        NotificationType.SUCCESS,
        `${cpState.value} has been copied to clipboard.`,
        'tabler:circle-check',
        3000,
      );
    }
  }, [cpState]);

  return (
    <div className={styles.color_picker}>
      <div className={styles.color_preview} style={{ backgroundColor: `#${selectedColor}` }} />
      <div className={styles.color_code} onClick={handleCopyAction}>
        #{selectedColor}
      </div>
      <HSegmentedControl
        options={[
          { label: 'RGB', value: 'rgb' },
          { label: 'HSL', value: 'hsl' },
          { label: 'Input', value: 'input' },
        ]}
        value={pickMode}
        onChange={(value) => setMode(value as 'rgb' | 'hsl' | 'input')}
      />
      {isEqual(pickMode, 'rgb') && (
        <RGBAssemble color={selectedColor} onChange={handleColorSelect} />
      )}
      {isEqual(pickMode, 'hsl') && (
        <HslAssemble color={selectedColor} onChange={handleColorSelect} />
      )}
      {isEqual(pickMode, 'input') && (
        <ColorComponentInput color={selectedColor} onChange={handleColorSelect} />
      )}
    </div>
  );
}
